/*
    This program demonstrates that each thread in a process
    has its own stack for its parameters and automatic variables,
    but all of the threads in a process share the same heap.

    After each thread prints out its information it goes into
    an infinite loop. Use a memory mapping program to observe
    the location of each stack in virtual memory. Notice how
    each stack has a guard page allocated to it. Also look at
    the very high end of virtual memory to see where the thread
    descriptor pages are stored by Windows.
*/
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>

HANDLE mutex; // Mutex handle

int main()
{
   //thread function prototype
   DWORD WINAPI threadFunction(LPVOID);

   // the following local variable is automatically created on main's stack
   char* stackVariable;

   printf("The address of main's automatic local variable is %p\n", &stackVariable);

   // now make the stack variable point to something in the heap
   stackVariable = malloc(sizeof(char));

   printf("The address of the dynamic (heap) variable is %p\n\n", stackVariable);

   // if you want to, you can pause the program so that there is time
   // to start the memory mapper and watch the thread stacks get created
   //Sleep(20*1000); // wait 20 seconds

   // create a Mutex object (in signaled state)
   mutex = CreateMutex(NULL, FALSE, NULL);

   // create three threads
   CreateThread(NULL, 0, threadFunction, NULL, 0, NULL);
   CreateThread(NULL, 0, threadFunction, NULL, 0, NULL);
   CreateThread(NULL, 0, threadFunction, NULL, 0, NULL);

   // go into an infinite loop that does not use any processor resources
   while(1) Sleep(1000);

   return 0;
}//main


DWORD WINAPI threadFunction(LPVOID threadParam)
{
   // the following local variable is automatically created on this thread's stack
   char* stackVariable;

   // now make the stack variable point to something in the heap
   stackVariable = malloc(sizeof(char));

   WaitForSingleObject(mutex, INFINITE);  // try commenting this out
   printf("The address of this thread's parameter is %p\n", &threadParam);

   printf("The address of this thread's automatic local variable is %p\n", &stackVariable);

   printf("The address of the dynamic (heap) variable is %p\n\n", stackVariable);
   ReleaseMutex(mutex);                   // and comment this out also

   // go into an infinite loop that does not use any processor resources
   while(1) Sleep(1000);

   return 0;
}//threadFunction
