/*
   This program repeatedly allocates and deallocates pages of
   virtual memory (at virtual memory address 0x500000).

   Use a memory mapping program (such as VMmapper.exe) to watch
   this process's virtual memory map change.

   If you uncomment to the code that tries to read memory location
   0x500004, then you can see when that location is really available
   and when it is not.
*/
#include <windows.h>
#include <stdio.h>

void printError(char* functionName);

int main()
{
   char *address = (char*)0x05000000;
   LPVOID saveAdd;

   Sleep(5000);

   while( 1 )
   {
      // try to read memory location 0x5000004
      //printf( "%d\n", *(int*)(address + 4) );

      // 1. Reserve
      saveAdd = VirtualAlloc((LPVOID)address,
                             61440+4096,
                             MEM_RESERVE,
                             PAGE_READWRITE);
      if (saveAdd == 0)
         printError("VirtualAlloc");

      // try to read memory location 0x5000004
      //printf( "%d\n", *(int*)(address + 4) );

      Sleep(2000);

      // 2. Commit
      if (!VirtualAlloc((LPVOID)address,
                        4096,
                        MEM_COMMIT,
                        PAGE_READWRITE))
         printError("VirtualAlloc");

      Sleep(2000);

      // 3. Touch memory location 0x5000004
      printf( "%d\n", *(int*)(address + 4) );

      Sleep(2000);

      // 4. Free
      if (!VirtualFree((LPVOID)address,
                       0,
                       MEM_DECOMMIT))
         printError("VirtualFree");

      // try to read memory location 0x5000004
      //printf( "%d\n", *(int*)(address + 4) );

      Sleep(2000);

      // 5. Release
      if (!VirtualFree(saveAdd,
                       0,
                       MEM_RELEASE))
         printError("VirtualFree");

      Sleep(2000);
   }

   return 0;
}


/********************************************************************
    This method is used to print out useful error messages
    for Windows (but not C Library) function calls.
*/
void printError(char* functionName)
{
   LPVOID lpMsgBuf;
   int error_no;
   error_no = GetLastError();
   FormatMessage(
      FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
      NULL,
      error_no,
      MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
      (LPTSTR) &lpMsgBuf,
      0,
      NULL
   );
   // Display the string.
   fprintf(stderr, "\n%s failed on error %d: ", functionName, error_no);
   fprintf(stderr, (LPTSTR)lpMsgBuf);
   // Free the buffer.
   LocalFree( lpMsgBuf );
   ExitProcess(1);  // terminate the program
}
